<?php

/**
 * @file
 * Handles exporting of nodes.
 */

/**
 * Build an array of operations for exporting each node.
 */
function mover_export_nodes_operations($callback) {
  $result = db_query(db_rewrite_sql("SELECT n.nid FROM {node} n ORDER BY n.nid"));
  while ($n = db_fetch_object($result)) {
    $operations[] = array($callback, array($n->nid));
  }
  $operations[] = array('mover_export_make_tarball_process', array('nodes'));
  return $operations;
}

/**
 * Batch processing callback for exporting each node.
 */
function mover_export_nodes_process($nid, &$context) {
  $node = node_load($nid, NULL, TRUE);
  if ($node->nid) {
    $data = mover_export_node($node);

    // Write the exported file
    $directory = variable_get('mover_export_directory', '');
    $subdirectory = mover_get_directory_for_file($directory, 'nodes', $context);
    $filepath = $subdirectory .'/'. $node->nid .'.txt';
    file_put_contents($filepath, $data);

    $context['message'] = check_plain($node->title);
    $context['results'][] = check_plain($node->title);
  }
  else {
    $message = t('Unable to load NID @nid', array('@nid' => $nid));
    drupal_set_message($message);
    $context['message'] = check_plain($message);
    $context['results'][] = $nid;
  }
}

/**
 *  Converts a node object to plain text code.
 *  Set $format to some string if encoding should be handled by some module that
 *  will recognise the string.
 */
function mover_export_node($original_node, $iteration = 0, $format = NULL) {
  if (isset($original_node->nid)) {
    global $user;

    $node = drupal_clone($original_node);

    // Fix menu array
    $node->menu = mover_get_menu($original_node);

    // Add more data of node author
    //$account = user_load(array('uid' => $node->uid));
    //$node->user = $account;

    // Only export term IDs, term data is exported separately
    $new_taxonomy = array();
    if (is_array($node->taxonomy)) {
      foreach ($node->taxonomy as $term) {
        $new_taxonomy[] = $term->tid;
      }
    }
    $node->taxonomy = $new_taxonomy;

    // Normally there is only one URL alias, but Drupal allows multiple
    // aliases, so to protect against that edge case we'll export all
    // URL aliases.
    $node->aliases = array();
    $result = db_query("SELECT dst FROM {url_alias} WHERE src='%s'", 'node/'. $node->nid);
    while ($alias = db_fetch_object($result)) {
      $node->aliases[] = $alias->dst;
    }

    // Export any URL redirects to this node (including redirects that point to the URL aliases)
    $node->redirects = array();
    $aliases = $node->aliases;
    $aliases[] = 'node/'. $node->tid;
    $result = db_query("SELECT source FROM {path_redirect}
      WHERE redirect IN (". db_placeholders($aliases, 'varchar') .")", $aliases);
    while ($redirect = db_fetch_object($result)) {
      $node->redirects[] = $redirect->source;
    }

    // Search the node body and find any inline images
    $node->inline_files = array();

    $content = ($node->teaser == $node->body) ? $node->body : $node->teaser . $node->body;
    $filepaths = mover_find_inline_files($content);
    foreach ($filepaths as $filepath) {
      $directory = variable_get('mover_export_directory', '');
      $assets_path = $directory .'/files';
      $exported_file = _mover_get_export_data($filepath, $assets_path);
      if ($exported_file) {
        $filepath = _mover_clean_local_file_path($filepath);
        $node->inline_files[] = array(
          'filepath' => $filepath,
          'mover_exported_file' => $exported_file,
        );
      }
    }

    // Let other modules do special fixing up.
    // The function signature is: hook_mover_node_alter(&$node, $original_node, $method)
    // Where $method is 'export'
    drupal_alter('mover_node', $node, $original_node, 'export', $format);
    $node = mover_remove_recursion($node);
    $code = mover_object_encode($node, $iteration, $format);
    return $code;
  }
}

/**
 * Create a new menu entry with title, parent and weight exported from
 * another nodes menu. Returns NULL if the node has no menu title.
 */
function mover_get_menu($node) {
  // This will fetch the existing menu item if the node had one.
  node_invoke_nodeapi($node, 'prepare');

  // Only keep the values we care about.
  if (!empty($node->menu)) {

    // Store a copy of the old menu
    $old_menu = $node->menu;

    // Now fetch the defaults for a new menu entry.
    $node = NULL;
    node_invoke_nodeapi($node, 'prepare');

    // Make a list of values to attempt to copy.
    $menu_fields = array(
      'link_title', 'plid', 'menu_name', 'weight', // These should import properly always.
      'hidden', 'expanded', 'has_children', // These will only import properly on 'Save as a new node then edit' imports.
    );

    // Copy those fields from the old menu over the new menu defaults.
    foreach ($menu_fields as $menu_field) {
      $node->menu[$menu_field] = $old_menu[$menu_field];
    }

    // Return the menu.
    return $node->menu;
  }
}
