<?php

/**
 * @file
 * Handles exporting of path redirects.
 */

/**
 * Build an array of operations for exporting each redirect.
 */
function mover_export_redirects_operations($callback) {
  $result = db_query("SELECT rid FROM {path_redirect} ORDER BY rid");
  while ($r = db_fetch_object($result)) {
    $operations[] = array($callback, array($r->rid));
  }
  $operations[] = array('mover_export_make_tarball_process', array('redirects'));
  return $operations;
}

/**
 * Batch processing callback for exporting each redirect.
 */
function mover_export_redirects_process($rid, &$context) {
  $redirect = db_fetch_object(db_query("SELECT * FROM {path_redirect} WHERE rid=%d", $rid));
  if ($redirect->rid) {

    // Let other modules do special fixing up.
    // The function signature is: hook_mover_export_alter(&$redirect, $type)
    // Where $method is 'export'
    drupal_alter('mover_export', $redirect, 'redirect');
    $redirect = mover_remove_recursion($redirect);
    $data = mover_object_encode($redirect);

    // Write the exported file
    $directory = variable_get('mover_export_directory', '');
    $subdirectory = mover_get_directory_for_file($directory, 'redirects', $context);
    $filepath = $subdirectory .'/'. $redirect->rid .'.txt';
    file_put_contents($filepath, $data);

    $context['message'] = check_plain($redirect->source);
    $context['results'][] = check_plain($redirect->source);
  }
  else {
    $message = t('Unable to load redirect @rid', array('@rid' => $rid));
    drupal_set_message($message);
    $context['message'] = check_plain($message);
    $context['results'][] = $rid;
  }
}
