<?php

/**
 * @file
 * Handles exporting of users.
 */

/**
 * Build an array of operations for exporting each user.
 */
function mover_export_users_operations($callback) {
  $result = db_query("SELECT uid FROM {users} WHERE uid > 1 ORDER BY uid");
  while ($u = db_fetch_object($result)) {
    $operations[] = array($callback, array($u->uid));
  }
  $operations[] = array('mover_export_make_tarball_process', array('users'));
  return $operations;
}

/**
 * Batch processing callback for exporting each user.
 */
function mover_export_users_process($uid, &$context) {
  $account = user_load(array('uid' => $uid));
  if ($account->uid) {
    $directory = variable_get('mover_export_directory', '');
    $data = mover_export_user($account);

    // Write the exported file
    $directory = variable_get('mover_export_directory', '');
    $subdirectory = mover_get_directory_for_file($directory, 'users', $context);
    $filepath = $subdirectory .'/'. $account->uid .'.txt';
    file_put_contents($filepath, $data);

    $title = $account->name .' (UID '. $account->uid .')';
    $context['message'] = check_plain($title);
    $context['results'][] = check_plain($title);
  }
  else {
    $message = t('Unable to load user @uid', array('@uid' => $uid));
    drupal_set_message($message);
    $context['message'] = check_plain($message);
    $context['results'][] = $uid;
  }
}

/**
 *  Converts a user object to plain text code.
 *  Set $format to some string if encoding should be handled by some module that
 *  will recognise the string.
 */
function mover_export_user($original_user, $iteration = 0, $format = NULL) {
  if (isset($original_user->uid)) {
    global $user;

    $account = drupal_clone($original_user);

    drupal_set_title(t('Export of @title', array('@title' => $account->name)));

    // Let other modules do special fixing up.
    // The function signature is: hook_mover_user_alter(&$node, $original_node, $method)
    // Where $method is 'export'
    drupal_alter('mover_user', $account, $original_user, 'export', $format);
    $account = mover_remove_recursion($account);
    $code = mover_object_encode($account, $iteration, $format);
    return $code;
  }
}
